#ifndef GUN_EVENT_MANAGER_H
#define GUN_EVENT_MANAGER_H

#include <map>
#include <boost\concept_check.hpp>
#include <boost\ptr_container\ptr_map.hpp>

#include "Event.h"

namespace gun
{
	class EventManager
	{
	public:
		/*
		* Call subscribe with the template type being the event arg type of the event you want to subscribe to.
		* The parameter you pass in is a boost::function object which you create with the help of boost::bind.
		* boost:bind takes in a reference to the event handler function you want to be called when the event is invoked, 
		* the object subscribing to the event, and '_1' to indicate only one parameter will be used for the event,
		* i.e. the event args.
		*
		* The event args is a class you create that is derived from EventArgs.  It contains the parameters for the 
		* event, such as in the example below.
		*
		* The returned object will be an EventBind object.  Make sure you call Unsubscribe() on the EventBind in a
		* cleanup method or destructor of the subscriber class.  Note: Calling Unsubscribe() in the destructor may
		* not work.  I have encountered this bug, but haven't been able to fix it.
		* 
		* Example:
		*
		****Subscriber Class
		*
		*		collided_bind_ = event_manager->Subscribe<CollidedEventArgs>(
		*			boost::bind(&gun::DamageOnCollisionPart::Handle_Collided, this, _1));
		*		...
		*		void gun::DamageOnCollisionPart::Handle_Collided(CollidedEventArgs& args)
		*
		****CollidedEventArgs.h:
		*
		*	#ifndef GUN_COLLIDED_EVENT_ARGS_H
		*	#define GUN_COLLIDED_EVENT_ARGS_H
		*
		*	#include "EventArgs.h"
		*
		*	namespace gun
		*	{
		*		class GameObject;
		*
		*		class CollidedEventArgs : public EventArgs
		*		{
		*		public:
		*			GameObject* game_object1;
		*			GameObject* game_object2;
		*
		*			CollidedEventArgs(GameObject* game_object1, GameObject* game_object2);
		*
		*			std::string Details();
		*		};
		*	}
		*
		*	#endif
		*/
		template<class T>
		EventBind Subscribe(const boost::function<void(T&)>& event_handler)
		{
			BOOST_CONCEPT_ASSERT((boost::Convertible<T, EventArgs>));

			Event<T>* event = FindEvent<T>();

			return event->Subscribe(event_handler);
		}

		/*
		* Call Publish with the event args of the event you want to publish.
		*
		* Ex: event_manager_->Publish(CollidedEventArgs(gameObject1, gameObject2));
		*
		*/
		template<class T>
		void Publish(T& event_args)
		{
			BOOST_CONCEPT_ASSERT((boost::Convertible<T, EventArgs>));

			Event<T>* event = FindEvent<T>();
			event->Publish(event_args);
		}

	private:
		boost::ptr_map<int, EventBase> events_;

		template<class T>
		Event<T>* FindEvent()
		{
			BOOST_CONCEPT_ASSERT((boost::Convertible<T, EventArgs>));

			int key = typeid(T).hash_code();

			if(events_.find(key) == events_.end())
			{
				events_.insert(key, new Event<T>());
			}

			return static_cast<Event<T>*>(events_.find(key)->second);
		}
	};
}

#endif